%   *************************************
%   *SOLVER USING THE INERTIA MULTIPLIER*
%   *************************************


function new_newvalue=inertia_solver(newvalue,fluid,pipeline,gas,nodes_pr_dx,alfa)

k=pipeline.nodes;

tolerance=1e-6;
maxiter=100;

node_count=0;
slope_count=1;

C1=alfa*(gas.wavespeed/pipeline.area);
C2=(gas.wavespeed/2/pipeline.area)*(1/alfa-alfa);

%---Solving at the inner nodes---
for node=2:k-1
         node_count=node_count+1;
         if node_count==nodes_pr_dx
            s1=pipeline.s(slope_count);
            s2=pipeline.s(slope_count+1);
            slope_count=slope_count+1;
            node_count=0;
         else 
            s1=pipeline.s(slope_count);
            s2=s1;
         end
        
        err=inf; 
        M_c=fluid.massflow(node);
        M_p=newvalue(1,node);
        M_a=fluid.massflow(node-1);
        M_r=newvalue(1,node-1);
        M_b=fluid.massflow(node+1);
        M_s=newvalue(1,node+1);
        P_p=newvalue(2,node);
        P_a=fluid.pressure(node-1);
        P_b=fluid.pressure(node+1);

        fric=frictionfactor(M_p,gas,pipeline);
        B=gas.wavespeed;
        Const=(fric*gas.wavespeed^2*pipeline.dx/(pipeline.diameter*pipeline.area^2));
               
        if s1==0
            slope1=1;                           %slope-1 is horizontal
        else
            slope1=(exp(s1)-1)/s1;%slope-1 is non-horizontal
        end
        
        if s2==0
            slope2=1;                           %slope is horizontal
        else
            slope2=(exp(s2)-1)/s2;
        end
        iter3=0;

         while err>tolerance 
   
            f_xn(1,1)=C1*(M_p-M_a)+P_p-P_a+(Const/(P_p+P_a))*slope1*...
                    ((M_p+M_a)/2)*abs((M_p+M_a)/2)+...
                    (P_p^2/(P_p+P_a))*(exp(s1)-1)+C2*(M_r-M_a+M_p-M_c);
            
            f_xn(2,1)=C1*(M_p-M_b)-P_p+P_b+(Const/(P_p+P_b))*slope2*...
                    ((M_p+M_b)/2*abs((M_p+M_b)/2))+...
                    (P_b^2/(P_p+P_b)*(exp(s2)-1))+C2*(M_p-M_c+M_s-M_b);

            J(1,1)=C1+(Const/(P_a+P_p))*slope1*abs((M_p+M_a)/2)+C2;
            J(1,2)=1-(Const/(P_a+P_p))*slope1*((M_p-M_a)/2)*abs((M_p+M_a)/2)...
                    +((exp(s1)-1)*(2*P_p*P_a+P_p^2))/(P_p+P_a)^2;

            J(2,1)=C1+(Const/(P_p+P_b))*slope2*abs((M_p+M_b)/2)+C2;
            J(2,2)=-1-(Const/(P_p+P_b))*slope2*((M_p-M_a)/2)*abs((M_p+M_b)/2)...
                    -(P_b^2/(P_p+P_b)^2)*(exp(s2)-1);

             iter3=iter3+1;         
             
            newvalue(:,node)=[M_p;P_p]-J\f_xn;
            err=norm(newvalue(:,node)-[M_p;P_p]);
            
            M_p=newvalue(1,node);
            P_p=newvalue(2,node);

            if iter3>maxiter
                disp(sprintf('this did not converge in node: %d',node));
                break
            end

         end
%    end
end
new_newvalue=newvalue;
new_newvalue(:,1)=newvalue(:,1);
new_newvalue(:,k)=newvalue(:,k);
end
