%   **********************************
%   *The solver used to solve        *
%   *the transient problem using the *
%   *method of characteristics       *
%   **********************************

function fluid=solver(fluid,pipeline,gas,inletbc,outletbc,time,nodes_pr_dx,dt,alfa,massflow_corr)

k=pipeline.nodes;

tolerance=1e-6;
maxiter=100;

A=pipeline.area;
D=pipeline.diameter;
s=pipeline.s;

B=gas.wavespeed;
        
newvalue=[zeros(1,k);zeros(1,k)];

iterlist=zeros(1,k);
node_count=0;
slope_count=1;
for node=1:k
    %---inlet boundary---
    if node==1
        
        fr=frictionfactor(fluid.massflow(1),gas,pipeline);

        C=fr*B^2*pipeline.dx/D/A^2;
        if s(1)==0
            s_tempval=1;
        else
            s_tempval=(exp(s(1))-1)/s(1);
        end

        switch inletbc.type
            
            case 'pressure is given at inlet'
                
                newvalue(2,1)=inletbc.pressure;
                M_1=fluid.massflow(1);
                M_2=fluid.massflow(2);

                P_1=newvalue(2,1);
                P_2=fluid.pressure(2);

                err=inf;
                iter1=0;
                
                 
                while err>tolerance
                    newvalue(1,1)=M_1-((B/A)*(M_1-M_2)-P_1+P_2+...
                             (C/(P_1+P_2)*s_tempval*...
                             ((M_1+M_2)/2)*abs((M_1+M_2)/2)+(P_2^2/(P_2+P_1))*...
                             (exp(s(1))-1))/(C/(P_1+P_2))*...
                             s_tempval*abs((M_1+M_2)/2));      
                    err=norm(newvalue(1,1)-M_1);
                    M_1=newvalue(1,1);
                    iter1=iter1+1;
                    if iter1>maxiter
                        disp('we do not have convergence at inlet boundary');
                        break
                    end
                               iterlist(node)=iter1;    
                end
            

                    
            case 'massflow is given at inlet'    
                    if inletbc.case==1
                        % steady state
                        newvalue(1,1)=inletbc.massflow;
                    
                    elseif inletbc.case==2
                        % oscillating flow
                        if inletbc.osc_case==1
                            inletmass=interp1(linspace(1,length(inletbc.massflow),length(inletbc.massflow)),inletbc.massflow,1+time*dt/60);
                            newvalue(1,1)=inletmass*massflow_corr;
                        elseif inletbc.osc_case==2
                            newvalue(1,1)=inletbc.massflow+inletbc.oscmagnitude*sin(inletbc.oscperiod*time*dt);
                        end
                    elseif inletbc.case==3
                        % closing valve
                        if inletbc.close_case==1
%                             for i=1:timesteps
                                if time<ceil(inletbc.closingtime/dt)
                                    newvalue(1,1)=inletbc.massflow(1)*(1-time/ceil(inletbc.closingtime/dt));
                                    if newvalue(1,1)<0
                                        newvalue(1,1)=0;
                                    end
                                else          
                                    inletbc.massflow(1)=0;
                                end
                           
                                %newvalue(1,1)=inletbc.massflow(time);
                        elseif inletbc.close_case==2
                            newvalue(1,1)=massflow_corr*interp1(linspace(1,length(inletbc.massflow),length(inletbc.massflow)),inletbc.massflow,1+time*dt/60);
                        end
                    elseif inletbc.case==4
                        % opening valve
                        newvalue(1,1)=massflow_corr*interp1(linspace(1,length(inletbc.massflow),length(inletbc.massflow)),inletbc.massflow,1+time*dt/60);
                    end
                M_1=newvalue(1,1);
                M_2=fluid.massflow(2);

                P_1=fluid.pressure(1);
                P_2=fluid.pressure(2);

                err=inf;
                iter2=0;
                   
               while err>tolerance
                    newvalue(2,1)=P_1-((B/A)*(M_1-M_2)-P_1+P_2+C/(P_1+P_2)*...
                             s_tempval*(((M_1+M_2)/2)*abs((M_1+M_2)/2))+(P_2^2*(exp(s(1))-1))...
                             /(P_1+P_2))/(-1-(C/(P_1+P_2)^2)*s_tempval*((M_1+M_2)/2)*abs((M_1+M_2)/2)-...
                             (P_2^2)/(P_1+P_2)^2*(exp(s(1))-1));
                    
                    err=norm(newvalue(2,1)-P_1);
                    P_1=newvalue(2,1);
                    iter2=iter2+1;
                    if iter2>maxiter
                        disp('we do not have convergence at inlet boundary');
                        break
                    end
                                iterlist(node)=iter2;    
                end
                

        end
        
    %---Outlet boundary  
    elseif node==k
        
        if s(pipeline.initial_nodes-1)==0
                s_tempval2=0;
        else
                s_tempval2=(exp(s(pipeline.initial_nodes-1))-1)/s(pipeline.initial_nodes-1);
        end
        
        fr=frictionfactor(fluid.massflow(k),gas,pipeline);
        C=(fr*B^2*pipeline.dx)/(D*A^2);
        switch outletbc.type
    
            case 'pressure is given at outlet' 
                newvalue(2,k)=outletbc.pressure;
                M_k=fluid.massflow(k);
                M_k_1=fluid.massflow(k-1);

                P_k=newvalue(2,k);
                P_k_1=fluid.pressure(k-1);
                err=inf;
                iter1=0;
                while err>tolerance
                    newvalue(1,k)=M_k-((B/A)*(M_k-M_k_1)+P_k-P_k_1+C/(P_k+P_k_1)*...
                             s_tempval2*((M_k+M_k_1)/2)*abs((M_k+M_k_1)/2)+(P_k^2/....
                             (P_k+P_k_1))*(exp(s(pipeline.initial_nodes-1))-1))/...
                             (B/A+(C/(P_k+P_k_1))*...
                             s_tempval2*abs((M_k+M_k_1)/2));
                    err=norm(newvalue(1,k)-M_k);
                    M_k=newvalue(1,k);    
                    iter1=iter1+1;
                    if iter1>maxiter
                        disp('we do not have convergence at outlet boundary');
                        break
                    end  
                end

            case 'massflow is given at outlet'
                newvalue(1,k)=outletbc.massflow;  
                M_k=newvalue(1,k);
                M_k_1=fluid.massflow(k-1);

                P_k=fluid.pressure(k);
                P_k_1=fluid.pressure(k-1);
                err=inf;
                iter2=0;
                while err>tolerance
                    newvalue(2,k)=P_k-(B/A*(M_k-M_k_1)+P_k-P_k_1+(C/(P_k+P_k_1))*...
                            s_tempval2*((M_k+M_k_1)/2*abs((M_k+M_k_1)/2)+(P_k^2/....
                            (P_k_1+P_k))*(exp(s(pipeline.initial_nodes-1))-1)))/(1-(C/(P_k+P_k_1))*...
                            s_tempval2*((M_k+M_k_1)/2)*abs((M_k+M_k_1)/2)+...
                            ((2*P_k*P_k_1+P_k^2)/(P_k+P_k_1)^2)*(exp(s(pipeline.initial_nodes-1))-1));
                    err=norm(newvalue(2,k)-P_k);
                    P_k=newvalue(2,k);
                    iter2=iter2+1;
                    if iter2>maxiter
                        disp('we do not have convergence at outlet boundary');
                        break
                    end
                                iterlist(node)=iter2;    
                end
                
        end

    %---Nodes in the middle---  
    else
        node_count=node_count+1;
  
        if node_count==nodes_pr_dx
            s1=pipeline.s(slope_count);
            s2=pipeline.s(slope_count+1);
            slope_count=slope_count+1;
            node_count=0;
        else 
            s1=pipeline.s(slope_count);
            s2=s1;
        end
        
        iter3=0;
        err=inf; 
        M_p=fluid.massflow(node);
        M_a=fluid.massflow(node-1);
        M_b=fluid.massflow(node+1);
        P_p=fluid.pressure(node);
        P_a=fluid.pressure(node-1);
        P_b=fluid.pressure(node+1);
        fr=frictionfactor(M_p,gas,pipeline);
        %fb=fa;
        B=gas.wavespeed;
        C=fr*B^2*pipeline.dx/D/A^2;
                 
        if s1==0
            slope1=1;                           %slope-1 is horizontal
        else
            slope1=(exp(s1)-1)/s1;%slope-1 is non-horizontal
        end
        
        if s2==0
            slope2=1;                           %slope is horizontal
        else
            slope2=(exp(s2)-1)/s2;              %slope is non-horizontal  
        end
         while err>tolerance 
            f_xn(1,1)=(B/A)*(M_p-M_a)+P_p-P_a+C/(P_p+P_a)*slope1*...
                    ((M_p+M_a)/2)*abs((M_p+M_a)/2)+...
                    (P_p^2/(P_p+P_a))*(exp(s1)-1);
       
            f_xn(2,1)=(B/A)*(M_p-M_b)-P_p+P_b+C/(P_p+P_b)*slope2*...
                    ((M_p+M_b)/2*abs((M_p+M_b)/2))+...
                    (P_b^2/(P_p+P_b)*(exp(s2)-1));

            J(1,1)=(B/A)+C/(P_a+P_p)*slope1*abs((M_p+M_a)/2);
            J(1,2)=1-C/(P_a+P_p)*slope1*((M_p-M_a)/2)*abs((M_p+M_a)/2)...
                    +((exp(s1)-1)*(2*P_p*P_a+P_p^2))/(P_p+P_a)^2;
            J(2,1)=(B/A)+C/(P_p+P_b)*slope2*abs((M_p+M_b)/2);
            J(2,2)=-1-C/(P_p+P_b)*slope2*((M_p-M_a)/2)*abs((M_p+M_b)/2)...
                    -(P_b^2/(P_p+P_b)^2)*(exp(s2)-1);
             iter3=iter3+1;         

            newvalue(:,node)=[M_p;P_p]-J\f_xn;
            err=norm(newvalue(:,node)-[M_p;P_p]);
            
            M_p=newvalue(1,node);
            P_p=newvalue(2,node);

            if iter3>maxiter
                disp(sprintf('this did not converge in node: %d',node));
                break
            end
            iterlist(node)=iter3;  

         end
    end
end


%--------------------------------------------------------------------------
%   CORRECTING THE VALUES ACCORING TO THE INTERTIAL MULTIPLIER
%--------------------------------------------------------------------------
if alfa~=1
%***Correction of values***

newvalue=inertia_solver(newvalue,fluid,pipeline,gas,nodes_pr_dx,alfa);

fluid.massflow=newvalue(1,:);
fluid.pressure=newvalue(2,:);
else
%--------------------------------------------------------------------------
%  Asssigning the new value to fluid
%--------------------------------------------------------------------------

fluid.massflow=newvalue(1,:);
fluid.pressure=newvalue(2,:);
end
%--------------------------------------------------------------------------
%   function end
%--------------------------------------------------------------------------
end

