% Energy equation considered

% trying to implement the energy balance using CFD condition
% and longer timesteps, since the characteristic of the energy
% equation equals velocity U


% equation implemented equals the equation used in TGNet described 
% by Langelandsvik in his dissertation thesis

% clear all
% clc
function fluid=energy(fluid,energyCalcTime,flowrate_old,pressure_old)
global pipeline;
global gas;
%M=600;                      % mass flow rate
%temperature_old=fluid.temperature;
% make an equidistant temperature grid
deltaX=energyGrid(fluid,energyCalcTime);
energyGridProfile=linspace(0,pipeline.length,pipeline.length/deltaX);
% the corresponding flowrate and pressure grid.
%initialGridProfile=linspace(0,pipeline.length,pipeline.length/pipeline.DX);
ordGridProfile=linspace(0,pipeline.length,pipeline.length/pipeline.dx);
% Determine pipe conditions
T_new=fluid.temperature;    % give T_new an initial value

ordPipeCount=1;

for j=2:numel(energyGridProfile)   

    if energyGridProfile(j)>ordGridProfile(ordPipeCount)
        ordPipeCount=ordPipeCount+1;
    end

    %T_new(j)=fluid.temperature(ordPipeCount);
    burial=pipeline.burialType(ordPipeCount);
    tempDeltaX=(energyGridProfile(j)-ordGridProfile(ordPipeCount));
    % pressure gradient more precise
    newPressureGradX=(fluid.pressure(ordPipeCount)-...
        fluid.pressure(ordPipeCount+1))/pipeline.dx;
    pressure_new=fluid.pressure(ordPipeCount)+...
        newPressureGradX*(tempDeltaX);
    oldPressureGradX=(pressure_old(ordPipeCount)-...
        pressure_old(ordPipeCount+1))/pipeline.dx;
    pressure_old=pressure_old(ordPipeCount)+...
        oldPressureGradX*(tempDeltaX);
    
    % calculate the constant compressibility factor based on the old
    % temperature and the average pressure
    reducedPressure=0.5*(pressure_new+pressure_old)/gas.pc;
    reducedTemperature=T_new(j)/gas.Tc;
    constZfactor=compressibilityfactor(reducedPressure,reducedTemperature);
    
    velocityGrad_new=((fluid.massflow(ordPipeCount)/(rho*pipeline.area))-...
        (fluid.massflow(ordPipeCount+1))/(rho*pipeline.area))/pipeline.dx;
    velocityGrad_old=(flowrate_old(ordPipeCount)/(rho*pipeline.area)-...
        flowrate_old(ordPipeCount+1))/pipeline.dx;
    velocity_new=fluid.massflow(ordPipeCount)/(rho*pipeline.area)+...
        velocityGrad_new*(tempDeltaX);
    velocity_old=flowrate_old(ordPipeCount)/(rho_old*pipeline.area)+...
        velocityGrad_old*(tempDeltaX);
    velocity_avg=0.5*(velocity_old+velocity_new);
    
    temperatureGrad_old=((fluid.temperature(ordPipeCount)/(rho*pipeline.area))-...
        (fluid.temperature(ordPipeCount+1))/(rho.pipeline.area))/pipeline.dx;
    temperature_old=fluid.temperature(ordPipeCount)+temperatureGrad_old*...
        (tempDeltaX);
    
    
    T_err=inf;
    while T_err>1e-06
        T_temp=T_new(j);
        rho=pressure/(constZfactor*gas.R*T_new(j));
        rho_old=pressure_old/(constZfactor*gas.R*temperature_old(j));
        rho_avg=0.5*(rho+rho_old);
        U_Wtot=overallHeatTransfer(velocity_avg,burial,rho_avg);
        T_env=outsideTemperatures(burial);

        avgFlowRate=velocity_avg*rho_avg*pipeline.area;
        my=viscosity(T_new(j),rho_avg);
        f_energy=frictionfactor(avgFlowRate,rho_avg,my);
        %entities that changes as we iterate
        % determine outside temperature 
        [~, cv_new]=spesificHeat(T_new(j),pressure_new);
        [~, cv_old]=spesificHeat(temperature_old(j-1),pressure_old);

        T_new(j)=(rho_old*cv_old*temperature_old(j-1)-temperature_old(j-1)*rho*constZfactor*gas*...
            (velocity_new-velocity_old)+rho*((f_energy*deltaX)/(2*pipeline.diameter))...
            *velocity_avg+4*(U_Wtot/D)*deltaX*(temperature_old(j-1)+T_env))/(rho*cv_new-...
            0.5*rho*Z*gas.R*velocity_avg+(4*U_Wtot/pipeline.diameter)*deltaX*(temperature_old(j-1)+T_env));

        T_err=abs(T_temp-T_new(j));
    end
end
counter=1;
for jj=2:numel(ordGridProfile)
    for jjj=counter:numel(energyGridProfile)
        if ordGridProfile(jj)==energyGridProfile(jjj)
            fluid.temperature(jj)=T_new(jjj);
            counter=6;
        elseif ordGridProfile(jj)>energyGridProfile(jjj)
            fluid.temperature(jj)=0.5*(T_new(jjj-1)+T_new(jjj));
            counter=6;
        end
    end

end




% end function