% function to calculate the corresponding grid for the solution of the
% energy equation. This is done using the CFL condition and dx/dt=u as the
% characteristic  speed. Hence max[u_i]delta_t/delta_x<1 is stated as our
% limit

%       o-------o----- n+1
%       |   C=u/|     
%       |     / |     
%       |    /  |
%       |   /   |           deltaX=(u*t)
%       |  /    |
%       | /     |
%       |/      |
%       o-------o---- n
%       i-1     i       


function [deltaX]=energyGrid(fluid,energyCalcTime)
global pipeline;
global gas;
global dt;
U=zeros(pipeline.nodes,1);
for i=1:pipeline.nodes
%    pr=0; Tr=0; Z=0;
    pr=fluid.pressure(i)/gas.pc;
    Tr=fluid.temperature(i)/gas.Tc;
    Z=compressibilityfactor(pr,Tr);
    rho=fluid.pressure(i)/(Z*gas.R*fluid.temperature(i));
    U(i)=fluid.massflow(i)/(rho*pipeline.area);
end
%U=fluid.massflow/                             % velocity profile
% assuming thatvelocity distribution is denoted by U
deltaX_temp=max(U)*(dt*energyCalcTime);              % CFL condition
numEnergyNodes=ceil(pipeline.length/deltaX_temp)-1;  % #nodes in the solution of energy equation 
deltaX=pipeline.length/numEnergyNodes;              % new deltaX 


end