function [fluid]=energyEquationSolved(fluid,previousA,temperature)
global pipeline;
global gas;
global K;

fluid.temperature(1)=interp1(linspace(1,length(temperature.in),...
    length(temperature.in)),temperature.in,1+temperature.readingPrMin*fluid.time/60)+K;
% fluid.temperature(1)=fluid.temperature(1);
for node=2:pipeline.nodes
    velocity=0.5*(fluid.velocity(node)+previousA.velocity(node-1));
    density=0.5*(fluid.density(node)+previousA.density(node-1));
%     temperature=0.5*(fluid.temperature(node)+previousA.temperature(node-1));
    pressure=0.5*(fluid.pressure(node)+previousA.pressure(node-1));
    pipeSegment=ceil(node/(pipeline.nodesPrDx));
    burial=pipeline.burialType(pipeSegment);
    T_p=fluid.temperature(node);
    
    error=inf;
    iteration=1;
    while error>1e-5
    
        avgTemp=0.5*(T_p+previousA.temperature(node-1));
        lambda=frictionfactor(velocity,density,avgTemp);
        [cp, cv]=specificHeat(avgTemp,pressure);
        Tr=avgTemp/gas.Tc;
        pr=pressure/gas.pc;
        
        Z=compressibilityfactor(pr,Tr);
        T_env=outsideTemperatures(burial); 
        my=viscosity(avgTemp,density);
        U_Wtot=overallHeatTransfer(velocity,burial,density,my,cp);
%         U_Wtot=0;
%         lambda=0;
        [mu_JT dmudT]=JouleThomsonCoefficientCalc(avgTemp,pressure);

%         U_Wtot/((fluid.flowrate(node)+previousA.flowrate(node-1))/2)
%         1/cp
%         pi*pipeline.diameter*pipeline.dx
%         (avgTemp-T_env)
%         U_Wtot=0;
%         lambda=0;


    func=T_p-previousA.temperature(node-1)-(Z*gas.R*(T_p+previousA.temperature(node-1))/(2*cp))*...
        log(fluid.pressure(node)/previousA.pressure(node-1))-...
        mu_JT*(fluid.pressure(node)-previousA.pressure(node-1))-...
        (1/2)*(Z*gas.R)^2/cp*lambda/...
        (pipeline.diameter*pipeline.area)*((T_p+previousA.temperature(node-1))/...
        (fluid.pressure(node)+previousA.pressure(node-1)))^2*...
        ((fluid.flowrate(node)+previousA.flowrate(node-1))/2)^2*pipeline.dx+...        
        (2/cp)*U_Wtot/(fluid.flowrate(node)+previousA.flowrate(node-1))*pi*pipeline.diameter*...
        pipeline.dx*((T_p+previousA.temperature(node-1))/2-T_env);

    df_TpdTp=1-(Z*gas.R/(2*cp))*log(fluid.pressure(node)/previousA.pressure(node-1))+...
        -(dmudT)*fluid.pressure(node)+(Z*gas.R)^2/cp*lambda/...
        (pipeline.diameter*pipeline.area)*(T_p+previousA.temperature(node-1))/...
        (fluid.pressure(node)+previousA.pressure(node-1))^2*...
        ((fluid.flowrate(node)+previousA.flowrate(node-1))/2)^2*pipeline.dx+...
        (1/cp)*U_Wtot/(fluid.flowrate(node)+previousA.flowrate(node-1))...
        *pi*pipeline.diameter*pipeline.dx;
    newTemperature=T_p-func/df_TpdTp;
    error=norm(newTemperature-T_p);
    if iteration>100
        disp('no convergence')
        error
        break
    end
    T_p=newTemperature;
    iteration=iteration+1;
    end
    fluid.temperature(node)=T_p;
% U_Wtot

end

return