%   *******************************
%   * MAIN METHOD FOR NATURAL GAS *
%   * PIPELINE CALCULATION        *  
%   *******************************
clear all
clc
%***PRE-PROCESSING***
%-----------------------------------------------------
%   GLOBAL CONSTANTS 
g=9.81;     %   R-Universal gas constant
R=8314;     %   g-gravity
atm=1.013e5;%   one atmosphere(gauge pressure)
%-----------------------------------------------------
global date;
global sim_time;
%   READING DATA
global pipelinecase;
situation=input('pipeline situation: 1-exposed, 2-coated,3-partial buried : ');
if situation==1
    profile=xlsread('europipe2_fullyExposed.xls');  % reading data for thin and exposed pipeline
    pipelinecase='exposed';
elseif situation==2
    profile=xlsread('europipe2_coated.xls');          % reading data for coated pipeline
    pipelinecase='coated';
elseif situation==3
    profile=xlsread('europipe2_partiallyBuried.xls');  % reading data for thin but buried pipeline
    pipelinecase='buried';
else 
    disp('select either 1, 2 or 3')
    break
end

% reading flow data
global pReadingPrMin;
[flow_in pressure_in pressure_out h pReadingPrMin temperature]=select_testcase();
% adapting BC's

ReqFlowLength=60*h+1;                               % required of the flow input 
ReqPressLength=60*h*pReadingPrMin+1;     % required length of pressure input
PinRemove=numel(pressure_in)-(60*h*pReadingPrMin)-1;
PoutRemove=numel(pressure_out)-(60*h*pReadingPrMin)-1;
[pressure_in pressure_out flow_in temperature]=...
    boudaryAdapt(PinRemove,PoutRemove,ReqFlowLength,ReqPressLength,...
    pressure_in,pressure_out,flow_in,temperature,h);


%-----------------------------------------------------
%   PIPELINE PROPERTIES
%-----------------------------------------------------
global nodeMultiplicationfactor;
nodeMultiplicationfactor=1;
global pipeline;
%---pipeline constants---------------------------------
pipeline.diameter=1.016;                    % diameter of the pipe[m]
pipeline.area=0.25*pi*pipeline.diameter^2;  % area of the pipe[m2]
pipeline.roughness=3e-6;                    % Pipeline roughness[m]
pipeline.wallLayers=1;                      % number of layers in pipeline coating

%---pipeline properties--------------------------------
pipeline.height=profile(:,3);                     % height of the pipeline at each initial node[m]  
pipeline.length=profile(length(profile(:,1)),2);  % length of pipeline from K?rst?-Bokn[m]          
pipeline.initNodes=profile(length(profile(:,1)),1); % number of nodes from the original pipeline
% reqTotalNumberNodes= input('the total number of wanted nodes in calculation: ');
reqTotalNumberNodes=305*nodeMultiplicationfactor+1;
if reqTotalNumberNodes<62
    reqTotalNumberNodes=62;
end
pipeline.burialType=profile(:,4);
pipeline.nodesPrDx=round(reqTotalNumberNodes/pipeline.initNodes);
pipeline.DX=pipeline.length/(pipeline.initNodes-1);
pipeline.dx=pipeline.DX/pipeline.nodesPrDx;
pipeline.nodes=pipeline.nodesPrDx*(pipeline.initNodes-1)+1;
pipeline.dXprofile=linspace(0,pipeline.length,pipeline.nodes);
fprintf('total number of pipeline nodes: %3.4g\n', pipeline.nodes);

for i=1:(profile(length(profile(:,1)),1)-1)
    pipeline.alfa(i)=atan((pipeline.height(i+1)-pipeline.height(i))/pipeline.DX);
end

%-----------------------------------------------------
%   NATURAL GAS PROPERTIES
%-----------------------------------------------------
global gas;
gas.SG=0.65;                        % specific gravity of gas
gas.MW=18.01;                       % molecular weight
gas.R=R/gas.MW;                     % specific gas constant
gas.Tc=191;                         % critical temperature gas[K]  
gas.pc=46.4e5;                      % critical gas pressure[Pa] 

%-----------------------------------------------------
%   INITIAL CONDITIONS 
%-----------------------------------------------------
global K; K=273.15;                 % Kelvin
initial.p_in=pressure_in(1)*atm;
initial.temperature=temperature.in(1)+K;
initial.Tr=initial.temperature/gas.Tc;
initial.pr=initial.p_in/gas.pc;
initial.Z=compressibilityfactor(initial.pr,initial.Tr);
initial.rho=initial.p_in/(initial.Z*gas.R*initial.temperature);
initial.my=viscosity(initial.temperature,initial.rho);
initial.wavespeed=sqrt(initial.Z*gas.R*initial.temperature);

%-----------------------------------------------------
%   STEADY-STATE
%-----------------------------------------------------
%***steady state calculations*******
MSM3_D=flow_in(1);
global flowCorr; flowCorr=8.831;
steadyFlowRate=MSM3_D*flowCorr;
steadyVelocity=steadyFlowRate/(initial.rho*pipeline.area);
pipeline.s=2*g*pipeline.dx*sin(pipeline.alfa)./initial.wavespeed^2;

initial.pressure(1)=initial.p_in;
friction=frictionfactor(steadyVelocity,initial.rho,initial.temperature);
C=friction*initial.wavespeed^2*steadyFlowRate^2*pipeline.dx/(pipeline.diameter*pipeline.area^2);

%*****case 1*****
if pipeline.nodesPrDx==1
disp('only one node each measuring point')
    for node=2:pipeline.nodes
            s=pipeline.s(node-1);
        if s==0
            initial.pressure(node)=sqrt(initial.pressure(node-1)^2-C);
        else
            initial.pressure(node)=sqrt((initial.pressure(node-1)^2-....
                C*((exp(s)-1)./s))/exp(s));
        end
    end
else
    for node=2:pipeline.nodes
          for n=0:pipeline.initNodes-2
                if node>=(2+pipeline.nodesPrDx*n) && node<=(pipeline.nodesPrDx*(1+n))
                    s=pipeline.s(n+1);
                end
          end
        if s==0
            initial.pressure(node)=sqrt(initial.pressure(node-1)^2-C);
        else
            initial.pressure(node)=sqrt((initial.pressure(node-1)^2-....
                C*((exp(s)-1)./s))/exp(s));
        end
    end
end

%__________INITIAL PLOTS______________
%     figure(1)
%     subplot(2,1,2)
% %     hold on
%     
%     subplot(2,1,1)
%     title('PIPE PROFILE')
%     plot(linspace(0,pipeline.length,length(pipeline.height)),pipeline.height);
%     %clearaxis([0 12500 -100 150])
%     xlabel('pipeline length[m]')
%     ylabel('heightprofile[m]')    
%  
%     subplot(2,1,2)
%     title('PLOT OF INITIAL PRESSUREDISTRIBUTION')
%     plot(linspace(0,pipeline.length,pipeline.nodes),(initial.pressure)/10^5)
% 
%     xlabel('meter[m]')
%     ylabel('Pressure[Bar]')
%     
%     hold off
%-----------------------------------------------
% Initiating class FLUID
%-----------------------------------------------
global xScale;
xScale=linspace(0,pipeline.length,pipeline.nodes);
temperatureScale=linspace(0,pipeline.length,pipeline.nodes*2-1);

fluid.temperature=calcInitialTemperatureDistribution(temperature);
fluid.flowrate=ones(1,pipeline.nodes)*steadyFlowRate;  
fluid.pressure=initial.pressure;
 
for iter=1:pipeline.nodes
    redPressure=initial.pressure(iter)/gas.pc;
    redTemperature=fluid.temperature(iter)/gas.Tc;
    fluid.Z(iter)=compressibilityfactor(redPressure,redTemperature);
    fluid.wavespeed(iter)=sqrt(fluid.Z(iter)*gas.R*fluid.temperature(iter));
    fluid.density(iter)=initial.pressure(iter)/(gas.R*fluid.Z(iter)*fluid.temperature(iter));
    fluid.velocity(iter)=fluid.flowrate(iter)/(fluid.density(iter)*pipeline.area);
    [cp cv]=specificHeat(fluid.temperature(iter),fluid.pressure(iter));
    internalEnergy=cv*fluid.temperature(iter);
    fluid.enthalpy(iter)=internalEnergy+fluid.pressure(iter)/fluid.density(iter);
end
%oldFluid=fluid; % initiating old class to interpolate

%-----------------------------------------------------
% TIME EVALUATION
%-----------------------------------------------------
dt=pipeline.dx/max(fluid.wavespeed);
dt_old=0;
%-----time evaluation in seconds----------------------
seconds=h*3600;
timesteps=ceil(seconds*1/dt);
% time_new=0;
fluid.time=0;
time_old=0;
%-----------------------------------------------------
%   BOUNDARY CONDITION
%-----------------------------------------------------
global inletbc;
global outletbc;
inletbc.pressure=0;
outletbc.flowrate=0;
inletbc.flowrate=flow_in;
outletbc.pressure=pressure_out*atm;
outletbc.pressure(1)=initial.pressure(pipeline.nodes);
if sum(inletbc.flowrate )~=0
    inletbc.type='Flow rate  is given at inlet';
else 
    inletbc.type='pressure is given at inlet';
end
if sum(outletbc.flowrate) ~=0
    outletbc.type='Flow rate  is given at outlet';
else
    outletbc.type='pressure is given at outlet';
end
%-----------------------------------------------------
%***MAIN PROCESSING***
% creating class fluid
% saveFile=input('save results?[Y/N] ','s');
saveFile='N';
status=0;
while (status==0)
    if saveFile=='Y'
        nodeSave=input('Save result for each node number: ');
        FlowFileName=input('Save flow rate result as filename: ','s');
        PressureFileName=input('Save pressure result as filename: ','s'); 
        inltPressureFileName=input('Save the inletpressure as filename: ','s');
        %time_save=round(sec_save/dt);
        status=1;
    elseif saveFile=='N'
        status=1;
    else    
        status=0;
        saveFile=input('save results?[Y/N] ','s');
    end
end
minSave=round(60/dt);
if saveFile=='N'
    nodeSave=round(pipeline.nodes/62);
end
row=round(pipeline.nodes/nodeSave)+1;
numColumn=ceil(timesteps/(60/dt));
flowSave=zeros(numColumn,row);
pressureSave=zeros(numColumn,row);
temperatureSave=zeros(numColumn,row);
% fig4=figure(4);
% title('dynamic pressure at the inlet, calc vs measured')
dynamicPlot=input('Dynamic plot of results(aprrox 2x  time)? [Y/N]: ','s');
%---plot---------------------------------------------------
if dynamicPlot=='Y'
fig2=figure(2);
title('dynamic plot of inletpressure, flowrate and temperature')
subplot(2,2,1)
p1=plot(fluid.pressure(1)/atm);
xlabel('time')
ylabel('pressure[bar]')
subplot(2,2,2)
p2=plot(xScale,fluid.temperature-273.15);
xlabel('meter[m]')
ylabel('Temperature[oC]')
subplot(2,1,2)
p3=plot(xScale,fluid.flowrate);
ylabel('flow rate[kg/s]')
xlabel('meter[m]')
end
%-----------------------------------------------------------

% hold on
it=2;
pIndex=1;
% pressureSaveTime=round((60/pReadingPrMin/dt));
pressureAtInlet=zeros(1,seconds/(60/pReadingPrMin));
pressureAtInlet(1)=fluid.pressure(1);
%oldTemperature=0;
%fluid.temperature=zeros(1,pipeline.nodes);

%***Variables to solve the energy equation***
% iterator=1;
% for j=2:2:pipeline.nodes*2-1
%     previousA.velocity(j-1)=fluid.velocity(iterator);
%     previousA.velocity(j)=0.5*(fluid.velocity(iterator)+fluid.velocity(iterator+1));
%     previousA.density(j-1)=fluid.density(iterator);
%     previousA.density(j)=0.5*(fluid.density(iterator)+fluid.density(iterator+1));
%     previousA.pressure(j-1)=fluid.pressure(iterator);
%     previousA.pressure(j)=0.5*(fluid.pressure(iterator)+fluid.pressure(iterator+1));
%     previousA.Z(j-1)=fluid.Z(iterator);
%     previousA.Z(j)=0.5*(fluid.Z(iterator)+fluid.Z(iterator+1));
%     iterator=iterator+1;
% end
previousA=fluid;
% previousA.velocity(pipeline.nodes*2-1)=fluid.velocity(pipeline.nodes);
% previousA.density(pipeline.nodes*2-1)=fluid.density(pipeline.nodes);
% previousA.pressure(pipeline.nodes*2-1)=fluid.pressure(pipeline.nodes);
% previousA.Z(pipeline.nodes*2-1)=fluid.Z(end);
% previousA.temperature=fluid.temperature;

previousA.time=0;
C0char_dx=0;
previousD.temperature=fluid.temperature;
previousD.density=previousA.density;
previousD.velocity=previousA.velocity;
previousD.pressure=previousA.pressure;
previousD.time=0;
%*******************************

% initiatin class  to interpolate
oldFluid.time=0;
oldFluid.pressure=fluid.pressure;
oldFluid.flowrate=fluid.flowrate;
%-------------------------------

iterator=1;
timeD=1;
mincount=0;
alfa=1;
tic
contd=0;
energyCalculations=1;
SecondIterator=1;
iterator2=1;
% break
% for iter=1:timesteps
while fluid.time<seconds

% for iter=1:timesteps
%     previousA.time=fluid.time;
    fluid.time=fluid.time+dt;
    fluid.time;
    fluid=solver(fluid,oldFluid,dt,time_old);
   
    tempDxAdd=fluid.velocity*dt;
    C0char_dx=C0char_dx+tempDxAdd;
    diffDx=pipeline.dx-max(C0char_dx);
%   ***Soling the energy equation***
    if iterator==1
    previousA.time=fluid.time;
    % at the first iteraton the Energy equation is not resolved due to 
    % the integration in time, and the fact that integration at time=0
    % will lead to unreliable results

    elseif (diffDx)<tempDxAdd
        
    %------------------------------------------------------------
    %     solving the energy equation
    [fluid]=energyEquationSolved(fluid,previousA,temperature);
    previousA=fluid;
    %------------------------------------------------------------
    
    %------------------------------------------------------------
    % Energy 2 function
    % [fluid previousA testdata]=energy2(fluid,previousA,previousD);
    %------------------------------------------------------------
    
    %------------------------------------------------------------
    % Enthalpy equation- STABLE
    %     [fluid previousA]=enthalpySolver(fluid,previousA,temperature);
    %------------------------------------------------------------
    %------------------------------------------------------------
    % Complete energy equation- UNSTABLE
    % [fluid previousA]=newEnergyEqSolved(fluid,previousA,temperature);
    %------------------------------------------------------------
    

            C0char_dx=0;
            iterator2=iterator2+1;
    end

    if iterator==100
        time_evaluator=toc;
        completed=fluid.time/seconds;
        remaining_time=1/completed*time_evaluator;
        remaining_min=round(remaining_time/60);    
        fprintf('Approximately time of calculation[min]: %g\n',remaining_min);
    end

        if iterator==1
             for iter=nodeSave:nodeSave:pipeline.nodes
                flowSave(1,1)=fluid.flowrate(1);
                flowSave(1,iter/nodeSave+1)=fluid.flowrate(iter);
                flowSave(1,row)=fluid.flowrate(pipeline.nodes);
                pressureSave(1,1)=fluid.pressure(1);
                pressureSave(1,iter/nodeSave+1)=fluid.pressure(iter);
                pressureSave(1,row)=fluid.pressure(pipeline.nodes);
                temperatureSave(1,1)=fluid.temperature(1);
                temperatureSave(1,iter/nodeSave+1)=fluid.temperature(iter);
                temperatureSave(1,end)=fluid.temperature(end);
             end
        end


        timeToFullMin=60-mod(fluid.time,60);
        if timeToFullMin<dt
            
            for iter=nodeSave:nodeSave:pipeline.nodes
                flowSave(it,1)=fluid.flowrate(1);
                flowSave(it,iter/nodeSave+1)=fluid.flowrate(iter);
                flowSave(it,row)=fluid.flowrate(pipeline.nodes);
                pressureSave(it,1)=fluid.pressure(1);
                pressureSave(it,iter/nodeSave+1)=fluid.pressure(iter);
                pressureSave(it,row)=fluid.pressure(pipeline.nodes);
                temperatureSave(it,1)=fluid.temperature(1);
                temperatureSave(it,iter/nodeSave+1)=fluid.temperature(iter);
                temperatureSave(it,end)=fluid.temperature(end);
            end
            it=it+1;
             minSim=round(fluid.time/60);
             fprintf('time in minutes: %3.4g\n', minSim)
             if minSim==sim_time*60;
                 disp('simulation is complete')
                 break
             end
        end
        
       SaveInletPressure=60/pReadingPrMin-mod(fluid.time,60/pReadingPrMin);
       if SaveInletPressure<dt
        
           pIndex=pIndex+1;
           pressureAtInlet(pIndex)=fluid.pressure(1);
       end
       if dynamicPlot=='Y'
        if iterator>1

                fig2;
                subplot(2,2,1)
                p1=plot(linspace(1,pIndex,pIndex),pressureAtInlet(1:pIndex)/atm,'.-',linspace(1,pIndex,pIndex),pressure_in(1:pIndex),'r');
                xlabel('time')
                ylabel('pressure[barg]')

                subplot(2,2,2)
                p2=plot(xScale,fluid.temperature-273.15);
                xlabel('meter[m]')
                ylabel('Temperature[oC]')
                subplot(2,1,2)
                p3=plot(xScale,fluid.flowrate);
                ylabel('flowrate[kg/s]')
                xlabel('meter[m]')
        end
       end
  
      %---------------------------------------------
      % evaluate the timestep for the next iteration
      dt=pipeline.dx/max(fluid.wavespeed);
      if iterator~=1
        time_old=oldFluid.time; 
      end  

        oldFluid.time=fluid.time;
        oldFluid.pressure=fluid.pressure;
        oldFluid.flowrate=fluid.flowrate;

      if dynamicPlot=='Y'
            pause(0.01);
            delete(p1)
            delete(p2)
            delete(p3)
      end 
        iterator=iterator+1;

end
    

%     save_flow=strcat('flow_',strcat(pipelinecase,'_',num2str(pipeline.nodes),'_nodes','.dat'));
%     save_pressure=strcat('pressure_',strcat(pipelinecase,'_',num2str(pipeline.nodes),'_nodes','.dat'));
%     save_temperature=strcat('temperature_',strcat(pipelinecase,'_',num2str(pipeline.nodes),'_nodes','.dat'));

% save flow_040022009_isotherm.dat flowSave -ASCII
% save save_040022009_isotherm.dat pressureSave -ASCII
% save save_040022009_isotherm.dat temperatureSave -ASCII  
   

if strcmp(date,'04.02.2009')==1
    filename=strcat('04022009_',pipelinecase,'.xls');
elseif strcmp(date,'04.04.2008')==1
    filename=strcat('04042008_',pipelinecase,'.xls');
elseif strcmp(date,'24.01.2008')==1
    filename=strcat('24012008_',pipelinecase,'.xls');
elseif strcmp(date,'29.12.2007')==1
    filename=strcat('29122007_',pipelinecase,'.xls');
elseif strcmp(date,'19.10.2007')==1
    filename=strcat('19102007_',pipelinecase,'.xls');
end


xlswrite(filename,flowSave,'Flow');
xlswrite(filename,pressureSave,'Pressure');
xlswrite(filename,temperatureSave,'Temperature');


figure(11)
surf(flowSave)
figure(12)
surf(pressureSave/atm)
figure(13)
surf(tempeartureSave-K)

%     mass_size=size(flowSave);
%     press_size=size(pressureSave);
% if saveFile=='Y'
%     save FlowFileName flowSave -ASCII
%     save PressureFileName pressureSave -ASCII
%     save inletPressureFileName pressureAtInlet -ASCII   
% end
% save FlowPressureJTonly.dat flowSave -ASCII
% xlabel('length pipeline, [m]')
% set(gca,'XTick',0:10:61)
% set(gca,'XTickLabel',(0:2000:12000))
% ylabel('time,[hh:mm]')
% set(gca,'YTick',1:60:241)
% set(gca,'YTickLabel',['14:00';'15:00';'16:00';'17:00';'18:00'])
% 
% zlabel('pressure, [bar]')
%  xlabel('length pipeline, [m]')
% set(gca,'XTick',0:10:61)
% set(gca,'XTickLabel',(0:2000:12000))
% ylabel('time,[hh:mm]')
% set(gca,'YTick',1:60:241)
% set(gca,'YTickLabel',['16:00';'17:00';'18:00';'19:00';'20:00'])
%  set(gca,'XTick',1:60:241)
% set(gca,'XTickLabel',['16:00';'17:00';'18:00';'19:00';'20:00'])
