function  [fluid previousA ]=newEnergyEqSolved(fluid,previousA,temperature)
% function [JT_new JT_old]=newEnergyEqSolved(fluid,previousA,temperature)
global K;
global gas;
global pipeline;
fluid.temperature(1)=interp1(linspace(1,length(temperature.in),...
    length(temperature.in)),temperature.in,1+temperature.readingPrMin*fluid.time/60)+K;
for i=2:pipeline.nodes
    pipeSegment=ceil(i/(pipeline.nodesPrDx));
    burial=pipeline.burialType(pipeSegment);
    T_env=outsideTemperatures(burial); 
    
%     U_avg=0.5*(fluid.velocity(i)+previousA.velocity(i-1));
%     rho=0.5*(fluid.density(i)+previousA.density(i-1));
    [cp_old, cv_old]=specificHeat(previousA.temperature(i-1),previousA.pressure(i-1));
    my_o=viscosity(previousA.temperature(i-1),previousA.density(i-1));
    oldU_Wtot=overallHeatTransfer(previousA.velocity(i-1),burial,...
        previousA.density(i-1),my_o,cp_old);
    fold=frictionfactor(previousA.velocity(i-1),previousA.density(i-1),previousA.temperature(i-1));
    DissipationA=previousA.density(i-1)*fold*previousA.velocity(i-1)^3*...
        previousA.time/(2*pipeline.diameter);
    HeatTransferA=4*oldU_Wtot*(previousA.temperature(i-1)-T_env)*...
        previousA.time/pipeline.diameter;
%     convTermA=previousA.density(i-1)*cp_old*previousA.temperature(i-1);
    Tpr=previousA.temperature(i-1)/gas.Tc;
    p_pr=previousA.pressure(i-1)/gas.pc;
    [Zmattar dZdT]=mattarZfunction(Tpr,p_pr);
    JTcoeff_old=gas.R*previousA.temperature(i-1)^2/(previousA.pressure(i-1)*cp_old)*dZdT;
    JTterm_old=(1+previousA.density(i-1)*cp_old*JTcoeff_old)*previousA.pressure(i-1);
    Tnew=fluid.temperature(i);
    T_tmp=fluid.temperature(i);
    error=inf;
    iter=1;
    while error>1e-5
        [cp_new, cv_new]=specificHeat(Tnew,fluid.pressure(i));
        my_n=viscosity(Tnew,fluid.density(i-1));
        newU_Wtot=overallHeatTransfer(fluid.velocity(i),burial,...
            fluid.density(i),my_n,cp_new);
        fnew=frictionfactor(fluid.velocity(i),fluid.density(i),Tnew);
        DissipationP=fluid.density(i)*fnew*fluid.velocity(i)^3*...
            fluid.time/(2*pipeline.diameter);
        
        HeatTransfer_tmp=4*newU_Wtot/pipeline.diameter;
        
        Tpr=Tnew/gas.Tc;
        p_pr=fluid.pressure(i)/gas.pc;
%         fprintf('sellected node: %3d',i)
        [Zmattar dZdT]=mattarZfunction(Tpr,p_pr);       
        JTcoeff_new=gas.R*T_tmp^2/(fluid.pressure(i)*cp_new)*dZdT;
%         JTcoeff_new
        dJT_dT=2*gas.R*T_tmp/(fluid.pressure(i)*cp_new)*dZdT;
        JTterm_new=(1+fluid.density(i)*cp_new*JTcoeff_new)*fluid.pressure(i);
        drho_dT=-(fluid.pressure(i)/(fluid.Z(i)*gas.R*T_tmp^2)+...
            fluid.pressure(i)/(fluid.Z(i)^2*gas.R*fluid.temperature(i))*...
            dZdT);
        Tnew=T_tmp-(fluid.density(i)*cp_new*T_tmp-previousA.density(i-1)*...
            cp_old*previousA.temperature(i-1)-JTterm_new+JTterm_old-DissipationP...
            +DissipationA+HeatTransfer_tmp-HeatTransferA)/(fluid.density(i)*cp_new...
            -(drho_dT*JTcoeff_new+fluid.density(i)*dJT_dT)*cp_new*fluid.pressure(i)...
            +HeatTransfer_tmp);
        
        
        
        
        
%         Tnew=(1/(fluid.density(i)*cp_new+HeatTransfer_tmp*fluid.time))*...
%             ((JTterm_new-JTterm_old)+(DissipationP-DissipationA)+...
%             (HeatTransferA+HeatTransfer_tmp*T_env*fluid.time)+convTermA);
        error=abs(Tnew-T_tmp);
        T_tmp=Tnew;
        if iter>100
            disp('no convergence')
            break
        end
        iter=iter+1;
    end
    fluid.temperature(i)=Tnew;
%     JT_old(i-1)=previousA.density(i-1)*cp_old*JTcoeff_old;
%     JT_new(i-1)=fluid.density(i)*cp_new*JTcoeff_new;
end
% figure(3)
% plot(fluid.temperature-previousA.temperature)
previousA=fluid;
end