%   **********************************
%   *The solver used to solve        *
%   *the transient problem using the *
%   *method of characteristics       *
%   **********************************
function fluid=solver(fluid,oldFluid,dt,time_old)
%-------------------------------------------------
%   set global variables
global pReadingPrMin;
global flowCorr
%global pipeline.nodesPrDx
global inletbc
global outletbc
% global dt
global gas
global pipeline
readDivide=60/pReadingPrMin;
tolerance=1e-6;
maxiter=100;   
newvalue=[zeros(1,pipeline.nodes);zeros(1,pipeline.nodes)];
nodeCount=0;
slopeCount=1;
for node=1:pipeline.nodes
    %---inlet boundary---
    if node==1
        %avgVelocity=0.5*(fluid.veloscity(1)+fluid.velocity(2));
        %avgDensity=0.5*(fluid.density(1)+fluid.density(2));
%         pr=fluid.pressure(1)/gas.pc;
%         Tr=fluid.temperature(1)/gas.Tc;
%         fluid.compFactor(1)=compressibilityfactor(pr,Tr);
%         speedOfSound=sqrt(fluid.compFactor(1)*gas.R*fluid.temperature(1));
%         C=fr*gas.wavespeed^2*pipeline.dx/pipeline.diameter/pipeline.area^2;

        fr=frictionfactor(fluid.velocity(1),fluid.density(1),fluid.temperature(1));
        C=fr*fluid.wavespeed(1)^2*pipeline.dx/(pipeline.diameter*pipeline.area^2);
        if pipeline.s(1)==0
            s_tempval=1;
        else
            s_tempval=(exp(pipeline.s(1))-1)/pipeline.s(1);
        end

        switch inletbc.type
            case 'pressure is given at inlet'     
                newvalue(2,1)=inletbc.pressure;
                M_1=fluid.flowrate(1);
                M_2=fluid.flowrate(2);

                P_1=newvalue(2,1);
                P_2=fluid.pressure(2);

                err=inf;
                iter1=0;
                while err>tolerance
                    newvalue(1,1)=M_1-((fluid.wavespeed(1)/pipeline.area)*(M_1-M_2)-P_1+P_2+...
                             (C/(P_1+P_2)*s_tempval*...
                             ((M_1+M_2)/2)*abs((M_1+M_2)/2)+(P_2^2/(P_2+P_1))*...
                             (exp(pipeline.s(1))-1))/(C/(P_1+P_2))*...
                             s_tempval*abs((M_1+M_2)/2));      
                    err=norm(newvalue(1,1)-M_1);
                    M_1=newvalue(1,1);
                    iter1=iter1+1;
                    if iter1>maxiter
                        disp('we do not have convergence at inlet boundary');
                        break
                    end   
                end
            

                    
            case 'Flow rate  is given at inlet'    

                newvalue(1,1)=flowCorr*interp1(linspace(1,length(inletbc.flowrate),...
                    length(inletbc.flowrate)),inletbc.flowrate,1+fluid.time/60);
                
%                 newvalue(1,1)=fluid.flowrate(1);
                M_1=newvalue(1,1);
                
                tr=fluid.time-(pipeline.dx-fluid.wavespeed(2)*dt)/fluid.wavespeed(2);
                M_2=interpolate(fluid.flowrate(2),oldFluid.flowrate(2),...
                    tr,fluid.time,oldFluid.time,time_old);
%                 M_2=fluid.flowrate(2);                
                P_1=fluid.pressure(1);
                P_2=interpolate(fluid.pressure(2),oldFluid.pressure(2),...
                    tr,fluid.time,oldFluid.time,time_old);
%                 P_2=fluid.pressure(2);

                err=inf;
                iter2=0;
                   
               while err>tolerance
                    newvalue(2,1)=P_1-((fluid.wavespeed(1)/pipeline.area)*(M_1-M_2)-P_1+P_2+C/(P_1+P_2)*...
                             s_tempval*(((M_1+M_2)/2)*abs((M_1+M_2)/2))+(P_2^2*(exp(pipeline.s(1))-1))...
                             /(P_1+P_2))/(-1-(C/(P_1+P_2)^2)*s_tempval*((M_1+M_2)/2)*abs((M_1+M_2)/2)-...
                             (P_2^2)/(P_1+P_2)^2*(exp(pipeline.s(1))-1));
                    
                    err=norm(newvalue(2,1)-P_1);
                    P_1=newvalue(2,1);
                    iter2=iter2+1;
                    if iter2>maxiter
                        disp('we do not have convergence at inlet boundary');
                        break
                    end
               end
        end
        
    %---Outlet boundary  
    elseif node==pipeline.nodes
        
        if pipeline.s(pipeline.initNodes-1)==0
                s_tempval2=0;
        else
                s_tempval2=(exp(pipeline.s(pipeline.initNodes-1))-1)/pipeline.s(pipeline.initNodes-1);
        end
        
%         fr=frictionfactor(fluid.flowrate(pipeline.nodes),gas.rho,gas.my);
        fr=frictionfactor(fluid.velocity(end),fluid.density(end),fluid.temperature(end));
        C=(fr*fluid.wavespeed(end)^2*pipeline.dx)/(pipeline.diameter*pipeline.area^2);
        switch outletbc.type
    
            case 'pressure is given at outlet' 

                newvalue(2,end)=interp1(linspace(1,length(outletbc.pressure),...
                    length(outletbc.pressure)),outletbc.pressure,1+fluid.time/readDivide);
                %                 newvalue(2,end)=fluid.pressure(end);
                 tr=fluid.time-(pipeline.dx-fluid.wavespeed(end-1)*dt)/fluid.wavespeed(end-1);
%                 newvalue(2,end)=fluid.pressure(end);
                 M_k=fluid.flowrate(end);
                M_k_1=interpolate(fluid.flowrate(end-1),oldFluid.flowrate(end-1),...
                    tr,fluid.time,oldFluid.time,time_old);
%                 M_k_1=fluid.flowrate(end-1);

                P_k=newvalue(2,end);
                P_k_1=interpolate(fluid.pressure(end-1),oldFluid.pressure(end-1),...
                    tr,fluid.time,oldFluid.time,time_old);

%                 P_k_1=fluid.pressure(end-1);
                err=inf;
                iter1=0;
                
                while err>tolerance
                    newvalue(1,pipeline.nodes)=M_k-((fluid.wavespeed(end)/pipeline.area)*(M_k-M_k_1)+P_k-P_k_1+C/(P_k+P_k_1)*...
                             s_tempval2*((M_k+M_k_1)/2)*abs((M_k+M_k_1)/2)+(P_k^2/....
                             (P_k+P_k_1))*(exp(pipeline.s(pipeline.initNodes-1))-1))/...
                             (fluid.wavespeed(end)/pipeline.area+(C/(P_k+P_k_1))*...
                             s_tempval2*abs((M_k+M_k_1)/2));
                    err=norm(newvalue(1,pipeline.nodes)-M_k);
                    M_k=newvalue(1,pipeline.nodes);    
                    iter1=iter1+1;
                    if iter1>maxiter
                        disp('we do not have convergence at outlet boundary');
                        break
                    end  
                end

            case 'Flow rate  is given at outlet'
                newvalue(1,pipeline.nodes)=outletbc.flowrate;  
                M_k=newvalue(1,pipeline.nodes);
                M_k_1=fluid.flowrate(pipeline.nodes-1);

                P_k=fluid.pressure(pipeline.nodes);
                P_k_1=fluid.pressure(pipeline.nodes-1);
                err=inf;
                iter2=0;
                while err>tolerance
                    newvalue(2,pipeline.nodes)=P_k-(fluid.wavespeed(end)/pipeline.area*(M_k-M_k_1)+P_k-P_k_1+(C/(P_k+P_k_1))*...
                            s_tempval2*((M_k+M_k_1)/2*abs((M_k+M_k_1)/2)+(P_k^2/....
                            (P_k_1+P_k))*(exp(pipeline.s(pipeline.initNodes-1))-1)))/(1-(C/(P_k+P_k_1))*...
                            s_tempval2*((M_k+M_k_1)/2)*abs((M_k+M_k_1)/2)+...
                            ((2*P_k*P_k_1+P_k^2)/(P_k+P_k_1)^2)*(exp(pipeline.s(pipeline.initNodes-1))-1));
                    err=norm(newvalue(2,pipeline.nodes)-P_k);
                    P_k=newvalue(2,pipeline.nodes);
                    iter2=iter2+1;
                    if iter2>maxiter
                        disp('we do not have convergence at outlet boundary');
                        break
                    end
                end
                
        end

    %---Nodes in the middle---  
    else
        nodeCount=nodeCount+1;
  
        if nodeCount==pipeline.nodesPrDx
            s1=pipeline.s(slopeCount);
            s2=pipeline.s(slopeCount+1);
            slopeCount=slopeCount+1;
            nodeCount=0;
        else 
            s1=pipeline.s(slopeCount);
            s2=s1;
        end
        
        iter3=0;
        err=inf;

        
        M_p=fluid.flowrate(node);
        M_a=interpolate(fluid.flowrate(node-1),oldFluid.flowrate(node-1),...
                    tr,fluid.time,oldFluid.time,time_old);
%         M_a=fluid.flowrate(node-1);
        M_b=interpolate(fluid.flowrate(node+1),oldFluid.flowrate(node+1),...
                    tr,fluid.time,oldFluid.time,time_old);
%         M_b=fluid.flowrate(node+1);
        P_p=fluid.pressure(node);
        P_a=interpolate(fluid.pressure(node-1),oldFluid.pressure(node-1),...
                    tr,fluid.time,oldFluid.time,time_old);
%         P_a=fluid.pressure(node-1);
        P_b=interpolate(fluid.pressure(node+1),oldFluid.pressure(node+1),...
                    tr,fluid.time,oldFluid.time,time_old);
%         P_b=fluid.pressure(node+1);
        
        fr=frictionfactor(fluid.velocity(node),fluid.density(node),fluid.temperature(node));
        %fb=fa;
        %B=gas.wavespeed;
        C=fr*fluid.wavespeed(node)^2*pipeline.dx/pipeline.diameter/pipeline.area^2;
                 
        if s1==0
            slope1=1;                           %slope-1 is horizontal
        else
            slope1=(exp(s1)-1)/s1;%slope-1 is non-horizontal
        end
        
        if s2==0
            slope2=1;                           %slope is horizontal
        else
            slope2=(exp(s2)-1)/s2;              %slope is non-horizontal  
        end
        tempConst=fluid.wavespeed(node)/pipeline.area;
         while err>tolerance 
            f_xn(1,1)=tempConst*(M_p-M_a)+P_p-P_a+C/(P_p+P_a)*slope1*...
                    ((M_p+M_a)/2)*abs((M_p+M_a)/2)+...
                    (P_p^2/(P_p+P_a))*(exp(s1)-1);
       
            f_xn(2,1)=tempConst*(M_p-M_b)-P_p+P_b+C/(P_p+P_b)*slope2*...
                    ((M_p+M_b)/2*abs((M_p+M_b)/2))+...
                    (P_b^2/(P_p+P_b)*(exp(s2)-1));

            J(1,1)=tempConst+C/(P_a+P_p)*slope1*abs((M_p+M_a)/2);
            J(1,2)=1-C/(P_a+P_p)*slope1*((M_p-M_a)/2)*abs((M_p+M_a)/2)...
                    +((exp(s1)-1)*(2*P_p*P_a+P_p^2))/(P_p+P_a)^2;
            J(2,1)=tempConst+C/(P_p+P_b)*slope2*abs((M_p+M_b)/2);
            J(2,2)=-1-C/(P_p+P_b)*slope2*((M_p-M_a)/2)*abs((M_p+M_b)/2)...
                    -(P_b^2/(P_p+P_b)^2)*(exp(s2)-1);
             iter3=iter3+1;         

            newvalue(:,node)=[M_p;P_p]-J\f_xn;
            err=norm(newvalue(:,node)-[M_p;P_p]);
            
            M_p=newvalue(1,node);
            P_p=newvalue(2,node);

            if iter3>maxiter
                fprintf('this did not converge in node: %d',node);
                break
            end
         end
    end
end

%--------------------------------------------------------------------------
%  Asssigning the new value to fluid
%--------------------------------------------------------------------------

fluid.flowrate=newvalue(1,:);
fluid.pressure=newvalue(2,:);
for j=1:pipeline.nodes
    pr=fluid.pressure(j)/gas.pc;
    Tr=fluid.temperature(j)/gas.Tc;
    fluid.Z(j)=compressibilityfactor(pr,Tr);
    fluid.wavespeed(j)=sqrt(fluid.Z(j)*gas.R*fluid.temperature(j));
    fluid.density(j)=fluid.pressure(j)/(fluid.Z(j)*gas.R*fluid.temperature(j));
    fluid.velocity(j)=fluid.flowrate(j)/(fluid.density(j)*pipeline.area);
end
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%   function end
%--------------------------------------------------------------------------
end

